/*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*
                              <resetlog.h>
DESCRIPTION

EXTERNALIZED FUNCTIONS

This software is contributed or developed by KYOCERA Corporation.
(C) 2015 KYOCERA Corporation
(C) 2016 KYOCERA Corporation
(C) 2017 KYOCERA Corporation

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License version 2 and
only version 2 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.

*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*/
#ifndef _RESET_LOG_H
#define _RESET_LOG_H

#define HEADER_VERION           "v4.0.1"
#define HEADER_VERION_SIZE      (8)

#ifdef  __KERNEL__      /* linux kernel */
#include <linux/kc_phymap.h>
#define MSM_KCJLOG_BASE         (MSM_UNINIT_RAM_BASE)

#else /*__KERNEL__*/    /* boot system */
#include <stdint.h>
#ifdef  SCL_UNINIT_RAM_BASE
#define MSM_KCJLOG_BASE         (SCL_UNINIT_RAM_BASE)
#endif/*UNINIT_RAM_BASE*/
#endif/*__KERNEL__*/


#ifdef  MSM_KCJLOG_BASE
#define SIZE_SMEM_ALLOC         (512)
#define SIZE_CONTROL_INFO       (1024)
#define SIZE_KERNEL_LOG         (512 * 1024)
#define SIZE_LOGCAT_MAIN        (512 * 1024)
#define SIZE_LOGCAT_SYSTEM      (512 * 1024)
#define SIZE_LOGCAT_EVENTS      (256 * 1024)
#define SIZE_LOGCAT_RADIO       (512 * 1024)
#define SIZE_SMEM_EVENT_LOG     ( 50 * 1024)
#define SIZE_MODEM_F3_LOG       (512 * 1024)
#define SIZE_MODEM_ERR_DATA_LOG ( 16 * 1024)
#define SIZE_MODEM_EXT_DATA_LOG ( 64 * 1024)

#define SIZE_OCIMEM             (16384)
#define SIZE_RPM_CODE           (131072)
#define SIZE_RPM_DATA           (65536)
#define SIZE_RPM_MSG            (20480)
#define SIZE_RESET_STATUS       (4)
#define SIZE_CPU_CONTEXT_INFO   (  1 * 1024)

#define SIZE_KERNEL_BOOT_LOG    (64 * 1024)
#define SIZE_TRUSTZONE_LOG      ( 4 * 1024)
#define SIZE_SBL1_BOOT_LOG      (16 + 64 + (4 * 1024))
#define SIZE_APPSBL_BOOT_LOG    (16 +      (8 * 1024))

#define ADDR_KCJ_RAM_CONSOLE    (MSM_KCJLOG_BASE + SIZE_CONTROL_INFO)

#define ADDR_CONTROL_INFO       (MSM_KCJLOG_BASE)
#define ADDR_KERNEL_LOG         (ADDR_CONTROL_INFO       + SIZE_CONTROL_INFO      )
#define ADDR_LOGCAT_MAIN        (ADDR_KERNEL_LOG         + SIZE_KERNEL_LOG        )
#define ADDR_LOGCAT_SYSTEM      (ADDR_LOGCAT_MAIN        + SIZE_LOGCAT_MAIN       )
#define ADDR_LOGCAT_EVENTS      (ADDR_LOGCAT_SYSTEM      + SIZE_LOGCAT_SYSTEM     )
#define ADDR_LOGCAT_RADIO       (ADDR_LOGCAT_EVENTS      + SIZE_LOGCAT_EVENTS     )
#define ADDR_SMEM_EVENT_LOG     (ADDR_LOGCAT_RADIO       + SIZE_LOGCAT_RADIO      )
#define ADDR_MODEM_F3_LOG       (ADDR_SMEM_EVENT_LOG     + SIZE_SMEM_EVENT_LOG    )
#define ADDR_MODEM_ERR_DATA_LOG (ADDR_MODEM_F3_LOG       + SIZE_MODEM_F3_LOG      )
#define ADDR_MODEM_EXT_DATA_LOG (ADDR_MODEM_ERR_DATA_LOG + SIZE_MODEM_ERR_DATA_LOG)
#define ADDR_CPU_CONTEXT_INFO   (ADDR_MODEM_EXT_DATA_LOG + SIZE_MODEM_EXT_DATA_LOG)

#define ADDR_USB_DUMP_START     (ADDR_CPU_CONTEXT_INFO   + SIZE_CPU_CONTEXT_INFO  )
#define ADDR_OCIMEM             (ADDR_USB_DUMP_START)
#define ADDR_RPM_CODE           (ADDR_OCIMEM             + SIZE_OCIMEM            )
#define ADDR_RPM_DATA           (ADDR_RPM_CODE           + SIZE_RPM_CODE          )
#define ADDR_RPM_MSG            (ADDR_RPM_DATA           + SIZE_RPM_DATA          )
#define ADDR_RESET_STATUS       (ADDR_RPM_MSG            + SIZE_RPM_MSG           )
#define ADDR_USB_DUMP_END       (ADDR_RESET_STATUS       + SIZE_RESET_STATUS      )

#define ADDR_KERNEL_BOOT_LOG    (ADDR_USB_DUMP_END)
#define ADDR_TRUSTZONE_LOG      (ADDR_KERNEL_BOOT_LOG    + SIZE_KERNEL_BOOT_LOG   )
#define ADDR_SBL1_BOOT_LOG      (ADDR_TRUSTZONE_LOG      + SIZE_TRUSTZONE_LOG     )
#define ADDR_APPSBL_BOOT_LOG    (ADDR_SBL1_BOOT_LOG      + SIZE_SBL1_BOOT_LOG     )

#endif/*MSM_KCJLOG_BASE*/

enum {
    LOGGER_INFO_MAIN,
    LOGGER_INFO_SYSTEM,
    LOGGER_INFO_EVENTS,
    LOGGER_INFO_RADIO,
    LOGGER_INFO_MAX,
};

#define CRASH_MAGIC_CODE         "KC ERROR"

#define CRASH_SYSTEM_KERNEL      "KERNEL"
#define CRASH_SYSTEM_MODEM       "MODEM"
#define CRASH_SYSTEM_PRONTO      "PRONTO"
#define CRASH_SYSTEM_ADSP        "ADSP"
#define CRASH_SYSTEM_VENUS       "VENUS"
#define CRASH_SYSTEM_ANDROID     "ANDROID"
#define CRASH_SYSTEM_UNKNOWN     "UNKNOWN"

#define CRASH_KIND_PANIC         "KERNEL PANIC"
#define CRASH_KIND_FATAL         "ERR FATAL"
#define CRASH_KIND_EXCEPTION     "EXCEPTION"
#define CRASH_KIND_WDOG_HW       "HW WATCH DOG"
#define CRASH_KIND_WDOG_SW       "SW WATCH DOG"
#define CRASH_KIND_SYS_SERVER    "SYSTEM SERVER CRASH"
#define CRASH_KIND_PWR_KEY       "PWR KEY"
#define CRASH_KIND_KDFS_REBOOT   "KDFS REBOOT"
#define CRASH_KIND_UNKNOWN       "UNKNOWN"


/* RAM_CONSOLE Contol */
typedef struct {
    uint32_t                    sig;
    uint32_t                    start;
    uint32_t                    size;
} ram_console_header_type;

typedef struct {
    ram_console_header_type     header;
    unsigned char               msg[1];
} ram_console_type;

/* Log Control */
typedef struct {
    uint32_t                    w_off;
    uint32_t                    head;
} logger_log_info;

#define MAGIC_CODE_SIZE         (16)
#define CRASH_SYSTEM_SIZE       (16)
#define CRASH_KIND_SIZE         (32)
#define CRASH_TIME_SIZE         (48)

#define VERSION_SIZE            (64)
#define MODEL_SIZE              (32)
#define CRASH_INFO_DATA_SIZE    (32)
#define PET_TIME_SIZE           (36)

#define BANNER_SIZE             (192)
#define PRODUCT_SIZE            (16)
#define SOFT_VERSION_SIZE       (32)
#define BASEBAND_VERSION_SIZE   (48)
#define BUILD_DATE_SIZE         (32)

#define MINFO_SIZE              (14)

typedef struct {
    unsigned char   linux_ver[VERSION_SIZE];
    unsigned char   modem_ver[VERSION_SIZE];
    unsigned char   model[MODEL_SIZE];
    uint32_t        reserved[40];
} kcy_info_type ;

typedef struct {
    unsigned char   linux_banner[BANNER_SIZE];
    unsigned char   product_number[PRODUCT_SIZE];
    unsigned char   software_version[SOFT_VERSION_SIZE];
    unsigned char   baseband_version[BASEBAND_VERSION_SIZE];
    unsigned char   android_build_date[BUILD_DATE_SIZE];
    unsigned char   modem_build_date[BUILD_DATE_SIZE];
} kcd_info_type ;

typedef union{
    kcy_info_type   kcy;
    kcd_info_type   kcd;
} kcj_info_type ;

typedef struct {
    unsigned char   magic_code[MAGIC_CODE_SIZE];
    unsigned char   crash_system[CRASH_SYSTEM_SIZE];
    unsigned char   crash_kind[CRASH_KIND_SIZE];
    unsigned char   crash_time[CRASH_TIME_SIZE];
    uint32_t        ver_format;
    uint32_t        padding0[3];
    kcj_info_type   kcj_info;
    unsigned char   crash_info_data[CRASH_INFO_DATA_SIZE];
    unsigned char   panic_info_data[CRASH_INFO_DATA_SIZE];
    logger_log_info info[LOGGER_INFO_MAX];
    uint32_t        reserved0[4];
    uint32_t        regsave_addr;
    uint32_t        regsave_addr_check;
    uint32_t        padding1[2];
    uint32_t        m_info[MINFO_SIZE];
    uint32_t        ocimem_size;
    uint32_t        rpm_code_size;
    uint32_t        rpm_data_size;
    uint32_t        rpm_msg_size;
    uint32_t        reset_status_size;
    uint32_t        pmic_pon_status_size;
    uint32_t        padding2[3];
    uint32_t        reset_info_addr;
    uint32_t        kernel_log_magic;
    uint32_t        kernel_log_start;
    uint32_t        kernel_log_size;
    uint32_t        kernel_log_reserved;
    uint32_t        reserved[76];
} ram_log_info_type;

#ifdef  __KERNEL__      /* linux kernel */
typedef struct {
    void __iomem                *addr_control_info;
    void __iomem                *addr_kcj_ram_console;
    void __iomem                *addr_kernel_log;
    void __iomem                *addr_logcat_main;
    void __iomem                *addr_logcat_system;
    void __iomem                *addr_logcat_events;
    void __iomem                *addr_logcat_radio;
    void __iomem                *addr_smem_event_log;
    void __iomem                *addr_modem_f3_log;
    void __iomem                *addr_modem_err_data_log;
    void __iomem                *addr_modem_ext_data_log;
} addr_offset_type;
#endif/*__KERNEL__*/

#endif /* _RESET_LOG_H */

